@@ -2,7 +2,6 @@ from keyword import iskeyword
 import re
 import sphinxify
 import typing
-import yaml
 
 from .hooks_datacfg import (
     HooksDataYaml,
@@ -17,7 +16,6 @@ from .hooks_datacfg import (
 from .generator_data import GeneratorData, MissingReporter
 from .mangle import trampoline_signature
 
-_missing = object()
 
 # TODO: this isn't the best solution
 def _gen_int_types():
@@ -29,7 +27,7 @@ def _gen_int_types():
     yield "uintmax_t"
 
 
-_int32_types = set(_gen_int_types())
+_int32_types = frozenset(_gen_int_types())
 
 
 _rvp_map = {
@@ -134,7 +132,8 @@ class Hooks:
         if iskeyword(name):
             return f"{name}_"
         if not name.isidentifier() and not is_operator:
-            raise ValueError(f"name {name!r} is not a valid identifier")
+            if not self.report_only:
+                raise ValueError(f"name {name!r} is not a valid identifier")
 
         return name
 
@@ -228,7 +227,9 @@ class Hooks:
             v["data"] = var_data
             self._add_type_caster(v["raw_type"])
 
-        data["type_caster_includes"] = self._get_type_caster_includes()
+        for _, u in header.using.items():
+            self._add_type_caster(u["raw_type"])
+
         data["class_hierarchy"] = self.class_hierarchy
         data["subpackages"] = self.subpackages
         data["x_has_operators"] = self.has_operators
@@ -243,8 +244,13 @@ class Hooks:
             self._add_subpackage(tmpl_data_d, tmpl_data)
             templates[k] = tmpl_data_d
 
+            for param in tmpl_data.params:
+                self._add_type_caster(param)
+
         data["templates"] = templates
 
+        data["type_caster_includes"] = self._get_type_caster_includes()
+
     def _function_hook(self, fn, data: FunctionData, internal: bool = False):
         """shared with methods/functions"""
 
@@ -530,6 +536,9 @@ class Hooks:
         if class_data.ignore:
             return
 
+        for _, u in cls["using"].items():
+            self._add_type_caster(u["raw_type"])
+
         for typename in class_data.force_type_casters:
             self._add_type_caster(typename)
 
